@@ -90,9 +90,15 @@ struct flock
 #define OPENHANDLE_FADVISE_BY_FD  141
 #define OPENHANDLE_SEEK_BY_FD     142
 #define OPENHANDLE_STATFS_BY_FH   143
+#define OPENHANDLE_GETXATTRS      144
+#define OPENHANDLE_SETXATTRS      145
+#define OPENHANDLE_REMOVEXATTRS   146
+#define OPENHANDLE_LISTXATTRS     147
+#define OPENHANDLE_FS_LOCATIONS   148
+#define OPENHANDLE_reserved       149
 #define OPENHANDLE_TRACE_ME       150
 #define OPENHANDLE_QUOTA          151
-#define OPENHANDLE_FS_LOCATIONS   152
+#define OPENHANDLE_MKNODE_BY_NAME 152
 
 struct trace_arg
 {
@@ -104,15 +110,8 @@ struct trace_arg
 int gpfs_ganesha(int op, void *oarg);
 
 #define OPENHANDLE_HANDLE_LEN 40
-#define OPENHANDLE_SHORT_HANDLE_LEN 32
 #define OPENHANDLE_KEY_LEN 28
-#define OPENHANDLE_VERSION 1
-
-/* gpfs_max_fh_size will be OPENHANDLE_SHORT_HANDLE_LEN if
- * short_file_handle is enabled. Otherwise, it is set to
- * OPENHANDLE_HANDLE_LEN.
- */
-extern int gpfs_max_fh_size;
+#define OPENHANDLE_VERSION 2
 
 struct xstat_cred_t
 {
@@ -570,7 +569,7 @@ struct create_name_arg
 {
     int mountdirfd;                 /* in     */
     struct gpfs_file_handle *dir_fh;/* in     */
-    uint32_t dev;                   /* in     */
+    uint32_t dev;                   /* in dev or posix flags */
     int mode;                       /* in     */
     int len;                        /* in     */
     const char *name;               /* in     */
@@ -668,6 +667,42 @@ struct xstat_arg
     uint32_t *expire_attr;
 };
 
+struct getxattr_arg {
+	int mountdirfd;
+	struct gpfs_file_handle *handle;
+	uint32_t name_len;
+	char *name;
+	uint32_t value_len;
+	void *value;
+};
+
+struct setxattr_arg {
+	int mountdirfd;
+	struct gpfs_file_handle *handle;
+	int type;
+	uint32_t name_len;
+	char *name;
+	uint32_t value_len;
+	void *value;
+};
+
+struct removexattr_arg {
+	int mountdirfd;
+	struct gpfs_file_handle *handle;
+	uint32_t name_len;
+	char *name;
+};
+
+struct listxattr_arg {
+	int mountdirfd;
+	struct gpfs_file_handle *handle;
+	uint64_t cookie;
+	uint64_t verifier;
+	uint32_t eof;
+	uint32_t name_len;
+	void *names;
+};
+
 struct fs_loc_arg {
     int mountdirfd;
     struct gpfs_file_handle *handle;
