@@ -245,13 +245,6 @@ static inline void init_shared_drc(void)
 		drc->xt.cachesz = drc->cachesz;
 		xp->cache =
 		    gsh_calloc(drc->cachesz, sizeof(struct opr_rbtree_node *));
-		if (unlikely(!xp->cache)) {
-			LogCrit(COMPONENT_DUPREQ,
-				"UDP DRC hash partition allocation failed (ix=%d)",
-				ix);
-			drc->cachesz = 0;
-			break;
-		}
 	}
 }
 
@@ -262,26 +255,12 @@ void dupreq2_pkginit(void)
 {
 	int code __attribute__ ((unused)) = 0;
 
-	dupreq_pool = pool_init("Duplicate Request Pool",
-				sizeof(dupreq_entry_t),
-				pool_basic_substrate, NULL, NULL, NULL);
-	if (unlikely(!(dupreq_pool)))
-		LogFatal(COMPONENT_INIT,
-			 "Error while allocating duplicate request pool");
+	dupreq_pool =
+	    pool_basic_init("Duplicate Request Pool", sizeof(dupreq_entry_t));
 
-	nfs_res_pool = pool_init("nfs_res_t pool", sizeof(nfs_res_t),
-				 pool_basic_substrate,
-				 NULL, NULL, NULL);
-	if (unlikely(!(nfs_res_pool)))
-		LogFatal(COMPONENT_INIT,
-			 "Error while allocating nfs_res_t pool");
+	nfs_res_pool = pool_basic_init("nfs_res_t pool", sizeof(nfs_res_t));
 
-	tcp_drc_pool = pool_init("TCP DRC Pool", sizeof(drc_t),
-				 pool_basic_substrate,
-				 NULL, NULL, NULL);
-	if (unlikely(!(tcp_drc_pool)))
-		LogFatal(COMPONENT_INIT,
-			 "Error while allocating TCP DRC pool");
+	tcp_drc_pool = pool_basic_init("TCP DRC Pool", sizeof(drc_t));
 
 	drc_st = gsh_calloc(1, sizeof(struct drc_st));
 
@@ -356,14 +335,9 @@ static inline enum drc_type get_drc_type(struct svc_req *req)
  */
 static inline drc_t *alloc_tcp_drc(enum drc_type dtype)
 {
-	drc_t *drc = pool_alloc(tcp_drc_pool, NULL);
+	drc_t *drc = pool_alloc(tcp_drc_pool);
 	int ix, code __attribute__ ((unused)) = 0;
 
-	if (unlikely(!drc)) {
-		LogCrit(COMPONENT_DUPREQ, "alloc TCP DRC failed");
-		goto out;
-	}
-
 	drc->type = dtype;	/* DRC_TCP_V3 or DRC_TCP_V4 */
 	drc->refcnt = 0;
 	drc->retwnd = 0;
@@ -394,16 +368,8 @@ static inline drc_t *alloc_tcp_drc(enum drc_type dtype)
 		drc->xt.cachesz = drc->cachesz;
 		xp->cache =
 		    gsh_calloc(drc->cachesz, sizeof(struct opr_rbtree_node *));
-		if (unlikely(!xp->cache)) {
-			LogCrit(COMPONENT_DUPREQ,
-				"TCP DRC hash partition allocation failed (ix=%d)",
-				ix);
-			drc->cachesz = 0;
-			break;
-		}
 	}
 
- out:
 	return drc;
 }
 
@@ -416,8 +382,12 @@ static inline drc_t *alloc_tcp_drc(enum drc_type dtype)
  */
 static inline void free_tcp_drc(drc_t *drc)
 {
-	if (drc->xt.tree[0].cache)
-		gsh_free(drc->xt.tree[0].cache);
+	int ix;
+
+	for (ix = 0; ix < drc->npart; ++ix) {
+		if (drc->xt.tree[ix].cache)
+			gsh_free(drc->xt.tree[ix].cache);
+	}
 	PTHREAD_MUTEX_destroy(&drc->mtx);
 	LogFullDebug(COMPONENT_DUPREQ, "free TCP drc %p", drc);
 	pool_free(tcp_drc_pool, drc);
@@ -722,9 +692,11 @@ static inline const nfs_function_desc_t *nfs_dupreq_func(dupreq_entry_t *dv)
 
 	if (dv->hin.rq_prog == nfs_param.core_param.program[P_NFS]) {
 		switch (dv->hin.rq_vers) {
+#ifdef _USE_NFS3
 		case NFS_V3:
 			func = &nfs3_func_desc[dv->hin.rq_proc];
 			break;
+#endif /* _USE_NFS3 */
 		case NFS_V4:
 			func = &nfs4_func_desc[dv->hin.rq_proc];
 			break;
@@ -749,12 +721,14 @@ static inline const nfs_function_desc_t *nfs_dupreq_func(dupreq_entry_t *dv)
 				 (int)dv->hin.rq_vers);
 			break;
 		}
+#ifdef _USE_NLM
 	} else if (dv->hin.rq_prog == nfs_param.core_param.program[P_NLM]) {
 		switch (dv->hin.rq_vers) {
 		case NLM4_VERS:
 			func = &nlm4_func_desc[dv->hin.rq_proc];
 			break;
 		}
+#endif /* _USE_NLM */
 	} else if (dv->hin.rq_prog == nfs_param.core_param.program[P_RQUOTA]) {
 		switch (dv->hin.rq_vers) {
 		case RQUOTAVERS:
@@ -786,15 +760,10 @@ static inline dupreq_entry_t *alloc_dupreq(void)
 {
 	dupreq_entry_t *dv;
 
-	dv = pool_alloc(dupreq_pool, NULL);
-	if (!dv) {
-		LogCrit(COMPONENT_DUPREQ, "alloc dupreq_entry_t failed");
-		goto out;
-	}
-	memset(dv, 0, sizeof(dupreq_entry_t));	/* XXX pool_zalloc */
+	dv = pool_alloc(dupreq_pool);
 	gsh_mutex_init(&dv->mtx, NULL);
 	TAILQ_INIT_ENTRY(dv, fifo_q);
-out:
+
 	return dv;
 }
 
@@ -973,11 +942,6 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs,
 	}
 
 	dk = alloc_dupreq();
-	if (dk == NULL) {
-		release_dk = false;
-		status = DUPREQ_ERROR;
-		goto release_dk;
-	}
 
 	dk->hin.drc = drc;	/* trans. call path ref to dv */
 
