@@ -75,7 +75,8 @@ uint32_t root_op_export_set = EXPORT_OPTION_SQUASH_TYPES |
 
 void squash_setattr(struct attrlist *attr)
 {
-	if (attr->mask & ATTR_OWNER) {
+	if (attr->mask & ATTR_OWNER &&
+	    op_ctx->export_perms->anonymous_uid != 0) {
 		if (op_ctx->export_perms->options &
 		    EXPORT_OPTION_ALL_ANONYMOUS)
 			attr->owner = op_ctx->export_perms->anonymous_uid;
@@ -85,7 +86,8 @@ void squash_setattr(struct attrlist *attr)
 			attr->owner = op_ctx->export_perms->anonymous_uid;
 	}
 
-	if (attr->mask & ATTR_GROUP) {
+	if (attr->mask & ATTR_GROUP &&
+	    op_ctx->export_perms->anonymous_gid != 0) {
 		/* If all squashed, then always squash the owner_group.
 		 *
 		 * If root squashed, then squash owner_group if
@@ -114,6 +116,14 @@ void squash_setattr(struct attrlist *attr)
 bool nfs_compare_clientcred(nfs_client_cred_t *cred1,
 			    nfs_client_cred_t *cred2)
 {
+#ifdef _HAVE_GSSAPI
+	gss_name_t cred1_cred_name;
+	gss_name_t cred2_cred_name;
+	OM_uint32 maj_stat, min_stat;
+	int status;
+#endif
+
+
 	if (cred1 == NULL)
 		return false;
 	if (cred2 == NULL)
@@ -132,6 +142,35 @@ bool nfs_compare_clientcred(nfs_client_cred_t *cred1,
 			return false;
 		break;
 
+#ifdef _HAVE_GSSAPI
+	case RPCSEC_GSS:
+		maj_stat = gss_inquire_context(&min_stat,
+			cred1->auth_union.auth_gss.gss_context_id,
+			&cred1_cred_name, NULL, NULL, NULL, NULL, NULL, NULL);
+
+		if (maj_stat != GSS_S_COMPLETE &&
+		    maj_stat != GSS_S_CONTEXT_EXPIRED)
+			return false;
+
+		 maj_stat = gss_inquire_context(&min_stat,
+			cred2->auth_union.auth_gss.gss_context_id,
+			&cred2_cred_name, NULL, NULL, NULL, NULL, NULL, NULL);
+
+		if (maj_stat != GSS_S_COMPLETE &&
+		    maj_stat != GSS_S_CONTEXT_EXPIRED)
+			return false;
+
+		maj_stat = gss_compare_name(&min_stat, cred1_cred_name,
+					    cred2_cred_name, &status);
+		if (maj_stat != GSS_S_COMPLETE)
+			return false;
+
+		if (status == 0)
+			return false;
+
+		break;
+#endif
+
 	default:
 		if (memcmp
 		    (&cred1->auth_union, &cred2->auth_union, cred1->length))
@@ -393,12 +432,6 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 					gsh_malloc(op_ctx->creds->caller_glen *
 						   sizeof(gid_t));
 
-				if ((*garray_copy) == NULL) {
-					LogCrit(COMPONENT_DISPATCH,
-						"Attempt to sqaush caller_garray failed - no memory");
-					return NFS4ERR_ACCESS;
-				}
-
 				memcpy((*garray_copy),
 				       op_ctx->creds->caller_garray,
 				       op_ctx->creds->caller_glen *
@@ -452,8 +485,8 @@ void init_credentials(void)
 {
 	memset(op_ctx->creds, 0, sizeof(*op_ctx->creds));
 	memset(&op_ctx->original_creds, 0, sizeof(op_ctx->original_creds));
-	op_ctx->creds->caller_uid = (uid_t) ANON_UID;
-	op_ctx->creds->caller_gid = (gid_t) ANON_GID;
+	op_ctx->creds->caller_uid = op_ctx->export_perms->anonymous_uid;
+	op_ctx->creds->caller_gid = op_ctx->export_perms->anonymous_gid;
 	op_ctx->caller_gdata = NULL;
 	op_ctx->caller_garray_copy = NULL;
 	op_ctx->managed_garray_copy = NULL;
@@ -504,8 +537,8 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 	     EXPORT_OPTION_ACCESS_MASK) == 0) {
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"Access not allowed on Export_Id %d %s for client %s",
-			op_ctx->export->export_id,
-			op_ctx->export->fullpath,
+			op_ctx->ctx_export->export_id,
+			op_ctx->ctx_export->fullpath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -516,8 +549,8 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 	if ((op_ctx->export_perms->options & EXPORT_OPTION_NFSV4) == 0) {
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"NFS4 not allowed on Export_Id %d %s for client %s",
-			op_ctx->export->export_id,
-			op_ctx->export->fullpath,
+			op_ctx->ctx_export->export_id,
+			op_ctx->ctx_export->fullpath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -535,8 +568,8 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"NFS4 over %s not allowed on Export_Id %d %s for client %s",
 			xprt_type_to_str(xprt_type),
-			op_ctx->export->export_id,
-			op_ctx->export->fullpath,
+			op_ctx->ctx_export->export_id,
+			op_ctx->ctx_export->fullpath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -549,8 +582,8 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 	    && (port >= IPPORT_RESERVED)) {
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"Non-reserved Port %d is not allowed on Export_Id %d %s for client %s",
-			port, op_ctx->export->export_id,
-			op_ctx->export->fullpath,
+			port, op_ctx->ctx_export->export_id,
+			op_ctx->ctx_export->fullpath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -561,8 +594,8 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 	if (export_check_security(req) == false) {
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"NFS4 auth not allowed on Export_Id %d %s for client %s",
-			op_ctx->export->export_id,
-			op_ctx->export->fullpath,
+			op_ctx->ctx_export->export_id,
+			op_ctx->ctx_export->fullpath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -576,29 +609,29 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 /**
  * @brief Perform version independent ACCESS operation.
  *
- * This function wraps a call to cache_inode_access, determining the appropriate
+ * This function wraps a call to fsal_access, determining the appropriate
  * access_mask to use to check all the requested access bits. It requests the
  * allowed and denied access so that it can respond for each requested access
  * with a single access call.
  *
- * @param[in]  entry The cache inode entry to check access for
+ * @param[in]  obj Object handle to check access for
  * @param[in]  requested_access The ACCESS3 or ACCESS4 bits requested
  * @param[out] granted_access   The bits granted
  * @param[out] supported_access The bits supported for this inode
  *
- * @return cache inode status
- * @retval CACHE_INODE_SUCCESS all access was granted
- * @retval CACHE_INODE_FSAL_EACCESS one or more access bits were denied
- * @retval other values indicate a cache inode failure
+ * @return FSAL error
+ * @retval ERR_FSAL_NO_ERROR all access was granted
+ * @retval ERR_FSAL_ACCESS one or more access bits were denied
+ * @retval other values indicate a FSAL failure
  *
  */
 
-cache_inode_status_t nfs_access_op(cache_entry_t *entry,
+fsal_errors_t nfs_access_op(struct fsal_obj_handle *obj,
 				   uint32_t requested_access,
 				   uint32_t *granted_access,
 				   uint32_t *supported_access)
 {
-	cache_inode_status_t status;
+	fsal_status_t fsal_status;
 	fsal_accessflags_t access_mask;
 	fsal_accessflags_t access_allowed;
 	fsal_accessflags_t access_denied;
@@ -630,21 +663,21 @@ cache_inode_status_t nfs_access_op(cache_entry_t *entry,
 		access_mask |= FSAL_R_OK | FSAL_ACE_PERM_READ_DATA;
 
 	if (requested_access & ACCESS3_LOOKUP) {
-		if (entry->type == DIRECTORY)
+		if (obj->type == DIRECTORY)
 			access_mask |= FSAL_X_OK | FSAL_ACE_PERM_EXECUTE;
 		else
 			granted_mask &= ~ACCESS3_LOOKUP;
 	}
 
 	if (requested_access & ACCESS3_MODIFY) {
-		if (entry->type == DIRECTORY)
+		if (obj->type == DIRECTORY)
 			access_mask |= FSAL_W_OK | FSAL_ACE_PERM_DELETE_CHILD;
 		else
 			access_mask |= FSAL_W_OK | FSAL_ACE_PERM_WRITE_DATA;
 	}
 
 	if (requested_access & ACCESS3_EXTEND) {
-		if (entry->type == DIRECTORY)
+		if (obj->type == DIRECTORY)
 			access_mask |=
 			    FSAL_W_OK | FSAL_ACE_PERM_ADD_FILE |
 			    FSAL_ACE_PERM_ADD_SUBDIRECTORY;
@@ -653,14 +686,14 @@ cache_inode_status_t nfs_access_op(cache_entry_t *entry,
 	}
 
 	if (requested_access & ACCESS3_DELETE) {
-		if (entry->type == DIRECTORY)
+		if (obj->type == DIRECTORY)
 			access_mask |= FSAL_W_OK | FSAL_ACE_PERM_DELETE_CHILD;
 		else
 			granted_mask &= ~ACCESS3_DELETE;
 	}
 
 	if (requested_access & ACCESS3_EXECUTE) {
-		if (entry->type != DIRECTORY)
+		if (obj->type != DIRECTORY)
 			access_mask |= FSAL_X_OK | FSAL_ACE_PERM_EXECUTE;
 		else
 			granted_mask &= ~ACCESS3_EXECUTE;
@@ -677,11 +710,11 @@ cache_inode_status_t nfs_access_op(cache_entry_t *entry,
 		    FSAL_TEST_MASK(access_mask, FSAL_W_OK) ? 'w' : '-',
 		    FSAL_TEST_MASK(access_mask, FSAL_X_OK) ? 'x' : '-',
 		    FSAL_TEST_MASK(access_mask, FSAL_ACE_PERM_READ_DATA) ?
-			entry->type == DIRECTORY ?
+			obj->type == DIRECTORY ?
 			"list_dir" : "read_data" : "-",
 		    FSAL_TEST_MASK(access_mask,
 				   FSAL_ACE_PERM_WRITE_DATA) ?
-			entry->type == DIRECTORY ?
+			obj->type == DIRECTORY ?
 			"add_file" : "write_data" : "-",
 		    FSAL_TEST_MASK(access_mask, FSAL_ACE_PERM_EXECUTE) ?
 			"execute" : "-",
@@ -691,12 +724,11 @@ cache_inode_status_t nfs_access_op(cache_entry_t *entry,
 		    FSAL_TEST_MASK(access_mask, FSAL_ACE_PERM_DELETE_CHILD) ?
 			"delete_child" : "-");
 
-	status =
-	    cache_inode_access_sw(entry, access_mask, &access_allowed,
-				  &access_denied, true);
-
-	if (status == CACHE_INODE_SUCCESS ||
-	    status == CACHE_INODE_FSAL_EACCESS) {
+	fsal_status = obj->obj_ops.test_access(obj, access_mask,
+					       &access_allowed,
+					       &access_denied, false);
+	if (fsal_status.major == ERR_FSAL_NO_ERROR ||
+	    fsal_status.major == ERR_FSAL_ACCESS) {
 		/* Define granted access based on granted mode bits. */
 		if (access_allowed & FSAL_R_OK)
 			*granted_access |= ACCESS3_READ;
@@ -712,7 +744,7 @@ cache_inode_status_t nfs_access_op(cache_entry_t *entry,
 		if (access_allowed & FSAL_ACE_PERM_READ_DATA)
 			*granted_access |= ACCESS3_READ;
 
-		if (entry->type == DIRECTORY) {
+		if (obj->type == DIRECTORY) {
 			if (access_allowed & FSAL_ACE_PERM_DELETE_CHILD)
 				*granted_access |=
 				    ACCESS3_MODIFY | ACCESS3_DELETE;
@@ -772,5 +804,5 @@ cache_inode_status_t nfs_access_op(cache_entry_t *entry,
 					   ACCESS3_EXECUTE) ? "EXECUTE" : "-");
 	}
 
-	return status;
+	return fsal_status.major;
 }
